#include "mainwindow.h"

MainWindow::MainWindow(QMainWindow *parent) :
    QMainWindow(parent)
{
    _spreadsheet=new Spreadsheet();
    setCentralWidget(_spreadsheet);

    createActions();
    createToolBars();

    _findDialog=nullptr;

    setWindowIcon(QIcon(":/images/icon.png"));
    setWindowTitle(tr("Táblázatkezelő[*]"));
    setFile("");
    connect(_spreadsheet, SIGNAL(modified()),
            this, SLOT(spreadsheetModified()));

}

void MainWindow::createActions()
{
    _actionNew=new QAction(QIcon(":/images/new.png"),"New",this);
    _actionNew->setShortcut(tr("Ctrl+N"));

    _actionOpen=new QAction(QIcon(":/images/open.png"),"Open",this);
    _actionOpen->setShortcut(tr("Ctrl+O"));

    _actionSave=new QAction(QIcon(":/images/save.png"),"Save",this);
    _actionSave->setShortcut(tr("Ctrl+S"));

    _actionCut=new QAction(QIcon(":/images/cut.png"),"Cut",this);
    _actionCut->setShortcut(tr("Ctrl+X"));

    _actionCopy=new QAction(QIcon(":/images/copy.png"),"Copy",this);
    _actionCopy->setShortcut(tr("Ctrl+C"));

    _actionPaste=new QAction(QIcon(":/images/paste.png"),"Paste",this);
    _actionPaste->setShortcut(tr("Ctrl+V"));

    _actionSearch=new QAction(QIcon(":/images/find.png"),"Search",this);
    _actionSearch->setShortcut(tr("Ctrl+F"));

    _actionGoto=new QAction(QIcon(":/images/gotocell.png"),"Goto",this);
    _actionGoto->setShortcut(tr("Ctrl+G"));

    _actionDel = new QAction(tr("&Delete"), this);
    _actionDel->setShortcut(tr("Del"));
    _actionDel->setStatusTip(tr("Delete the current selection's "
                                  "contents"));

    connect(_actionNew,SIGNAL(triggered()),this,SLOT(newDocument()));
    connect(_actionOpen,SIGNAL(triggered()),this,SLOT(openDocument()));
    connect(_actionSave,SIGNAL(triggered()),this,SLOT(saveDocument()));
    connect(_actionCut,SIGNAL(triggered()),this,SLOT(cut()));
    connect(_actionCopy,SIGNAL(triggered()),this,SLOT(copy()));
    connect(_actionPaste,SIGNAL(triggered()),this,SLOT(paste()));
    connect(_actionSearch,SIGNAL(triggered()),this,SLOT(search()));
    connect(_actionGoto,SIGNAL(triggered()),this,SLOT(gotoCell()));
    connect(_actionDel,SIGNAL(triggered()),this,SLOT(del()));
}

void MainWindow::createToolBars()
{
    _fileToolBar=new QToolBar();
    _editToolBar=new QToolBar();
    _fileToolBar->addAction(_actionNew);
    _fileToolBar->addAction(_actionOpen);
    _fileToolBar->addAction(_actionSave);
    _editToolBar->addAction(_actionCut);
    _editToolBar->addAction(_actionCopy);
    _editToolBar->addAction(_actionPaste);
    _editToolBar->addAction(_actionSearch);
    _editToolBar->addAction(_actionGoto);
    _editToolBar->addAction(_actionDel);
    addToolBar(_fileToolBar);
    addToolBar(_editToolBar);
}



MainWindow::~MainWindow()
{
}

bool MainWindow::saveFile(const QString &fileName)
{
    if (!_spreadsheet->writeFile(fileName)) {
        //statusBar()->showMessage(tr("Saving canceled"), 2000);
        return false;
    }

    setFile(fileName);
    //statusBar()->showMessage(tr("File saved"), 2000);
    return true;
}

bool MainWindow::saveAsDocument()
{
    QString fileName = QFileDialog::getSaveFileName(this,
                               tr("Save Spreadsheet"), ".",
                               tr("Spreadsheet files (*.sp)"));
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
}

bool MainWindow::saveDocument()
{
    if (_file.isEmpty()) {
        return saveAsDocument();
    } else {
        return saveFile(_file);
    }
}

bool MainWindow::okToContinue()
{
    if (isWindowModified()) {
        int r = QMessageBox::warning(this, tr("Spreadsheet"),
                        tr("The document has been modified.\n"
                           "Do you want to save your changes?"),
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No,
                        QMessageBox::Cancel | QMessageBox::Escape);
        if (r == QMessageBox::Yes) {
            return saveDocument();
        } else if (r == QMessageBox::Cancel) {
            return false;
        }
    }
    return true;
}

void MainWindow::newDocument()
{
    if (okToContinue()) {
        _spreadsheet->clear();
        setFile("");
    }
}

bool MainWindow::loadFile(const QString &fileName)
{
    if (!_spreadsheet->readFile(fileName)) {
        //statusBar()->showMessage(tr("Loading canceled"), 2000);
        return false;
    }

    setFile(fileName);
    //statusBar()->showMessage(tr("File loaded"), 2000);
    return true;
}

void MainWindow::openDocument()
{
    if (okToContinue()) {
        QString fileName = QFileDialog::getOpenFileName(this,
                                   tr("Open Spreadsheet"), ".",
                                   tr("Spreadsheet files (*.sp)"));
        if (!fileName.isEmpty())
            loadFile(fileName);
    }
}

void MainWindow::copy()
{
    QList<QTableWidgetSelectionRange> ranges = _spreadsheet->selectedRanges();
    QTableWidgetSelectionRange range=ranges.first();

    QString str;
    for (int i = 0; i < range.rowCount(); ++i) {
        if (i > 0)
            str += "\n";
        for (int j = 0; j < range.columnCount(); ++j) {
            if (j > 0)
                str += "\t";
            str += _spreadsheet->getFormula(range.topRow() + i, range.leftColumn() + j);
        }
    }
    QApplication::clipboard()->setText(str);
}

void MainWindow::paste()
{
    QList<QTableWidgetSelectionRange> ranges = _spreadsheet->selectedRanges();
    QTableWidgetSelectionRange range=ranges.first();

    QString str = QApplication::clipboard()->text();
    QStringList rows = str.split('\n');
    int numRows = rows.count();
    int numColumns = rows.first().count('\t') + 1;

    if (range.rowCount() * range.columnCount() != 1
            && (range.rowCount() != numRows
                || range.columnCount() != numColumns)) {
        QMessageBox::information(this, tr("Spreadsheet"),
                tr("The information cannot be pasted because the copy "
                   "and paste areas aren't the same size."));
        return;
    }

    for (int i = 0; i < numRows; ++i) {
        QStringList columns = rows[i].split('\t');
        for (int j = 0; j < numColumns; ++j) {
            int row = range.topRow() + i;
            int column = range.leftColumn() + j;
            if (row < _spreadsheet->rowCount() && column < _spreadsheet->columnCount())
                _spreadsheet->setItem(row,column,new QTableWidgetItem(columns[j]));
        }
    }
}

void MainWindow::del()
{
    foreach (QTableWidgetItem *item, _spreadsheet->selectedItems())
        delete item;
}

void MainWindow::cut()
{
    copy();
    del();
}

void MainWindow::search()
{
    if (!_findDialog) {
        _findDialog = new FindDialog(this);
        connect(_findDialog, SIGNAL(findNext(const QString &,
                                            Qt::CaseSensitivity)),
                _spreadsheet, SLOT(findNext(const QString &,
                                           Qt::CaseSensitivity)));
        connect(_findDialog, SIGNAL(findPrevious(const QString &,
                                                Qt::CaseSensitivity)),
                _spreadsheet, SLOT(findPrevious(const QString &,
                                               Qt::CaseSensitivity)));
    }

    _findDialog->show();
    _findDialog->activateWindow();

}

void MainWindow::gotoCell()
{
    GotoCellDialog dialog(this);
    if (dialog.exec()) {
        QString str = dialog.getText().toUpper();
        _spreadsheet->setCurrentCell(str.mid(1).toInt() - 1,
                                    str[0].unicode() - 'A');
    }
}

void MainWindow::spreadsheetModified()
{
    setWindowModified(true);
    //updateStatusBar();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (okToContinue()) {
        event->accept();
    } else {
        event->ignore();
    }
}

